@@ -4,7 +4,7 @@
  * Tcl file scanning: regular expression matching on lines of a file.  
  * Implements awk.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1997 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -13,37 +13,29 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXfilescan.c,v 8.8 1997/07/05 08:00:30 markd Exp $
+ * $Id$
  *-----------------------------------------------------------------------------
  */
 
 #include "tclExtdInt.h"
-
-/*
- * FIX: Does not support binary data.  Can't fix until the regexp facility
- * handles it.
- */
+#include "tclRegexp.h"
 
 /*
  * A scan context describes a collection of match patterns and commands,
  * along with a match default command to apply to a file on a scan.
  */
  
-#define CONTEXT_A_CASE_INSENSITIVE_FLAG 2
-#define MATCH_CASE_INSENSITIVE_FLAG 4
-
 typedef struct matchDef_t {
-    TclX_regexp         regExpInfo;
+    Tcl_RegExp          regExp;
+    Tcl_Obj            *regExpObj;
     Tcl_Obj            *command;
     struct matchDef_t  *nextMatchDefPtr;
-    short               matchflags;
 } matchDef_t;
 
 typedef struct scanContext_t {
     matchDef_t  *matchListHead;
     matchDef_t  *matchListTail;
     Tcl_Obj     *defaultAction;
-    short        flags;
     char         contextHandle [16];
     Tcl_Channel  copyFileChannel;
     int          fileOpen;
@@ -58,14 +50,13 @@ typedef struct {
     scanContext_t    *contextPtr;   /* Current scan context. */
     Tcl_Channel       channel;      /* The channel being scanned. */
     char             *line;         /* The line from the file. */
+    Tcl_UniChar      *uniLine;      /* UniCode (wide) char line. */
+    int               uniLineLen;
     off_t             offset;       /* The offset into the file. */
     long              bytesRead;    /* Number of translated bytes read.*/
     long              lineNum;      /* Current scanned line in the file. */
     matchDef_t       *matchPtr;     /* The current match, or NULL for the
                                        default. */
-    Tcl_SubMatchInfo  subMatchInfo; /* Information about the subexpressions
-                                       that matched, or NULL if this is the
-                                       default match. */
 } scanData_t;
 
 /*
@@ -114,8 +105,8 @@ static void
 ClearCopyFile _ANSI_ARGS_((scanContext_t *contextPtr));
 
 static int
-SetMatchInfoVar _ANSI_ARGS_((Tcl_Interp       *interp,
-                             scanData_t       *scanData));
+SetMatchInfoVar _ANSI_ARGS_((Tcl_Interp *interp,
+                             scanData_t *scanData));
 
 static int
 ScanFile _ANSI_ARGS_((Tcl_Interp    *interp,
@@ -151,7 +142,7 @@ CleanUpContext (scanTablePtr, contextPtr)
     matchDef_t  *matchPtr, *oldMatchPtr;
 
     for (matchPtr = contextPtr->matchListHead; matchPtr != NULL;) {
-        TclX_RegExpClean (&matchPtr->regExpInfo);
+        Tcl_DecrRefCount(matchPtr->regExpObj);
         if (matchPtr->command != NULL)
             Tcl_DecrRefCount (matchPtr->command);
         oldMatchPtr = matchPtr;
@@ -180,7 +171,6 @@ ScanContextCreate (interp, scanTablePtr)
     scanContext_t *contextPtr, **tableEntryPtr;
 
     contextPtr = (scanContext_t *) ckalloc (sizeof (scanContext_t));
-    contextPtr->flags = 0;
     contextPtr->matchListHead = NULL;
     contextPtr->matchListTail = NULL;
     contextPtr->defaultAction = NULL;
@@ -244,9 +234,9 @@ CopyFileCloseHandler (clientData)
  * SetCopyFileObj --
  *   Set the copy file handle for a context.
  * Parameters:
- *   o interp  n     (O) - The Tcl interpreter, errors are returned in result.
- *   o contextPtr    (I) - Pointer to the scan context.
- *   o fileHandleObj (I) - Object containing file handle of the copy file.
+ *   o interp - The Tcl interpreter, errors are returned in result.
+ *   o contextPtr - Pointer to the scan context.
+ *   o fileHandleObj - Object containing file handle of the copy file.
  * Returns:
  *   TCL_OK or TCL_ERROR.
  *-----------------------------------------------------------------------------
@@ -411,8 +401,6 @@ TclX_ScancontextObjCmd (clientData, interp, objc, objv)
  *
  *   Implements the TCL command:
  *         scanmatch ?-nocase? contexthandle ?regexp? command
- *
- *   This uses both Boyer_Moore and regular expressions matching.
  *-----------------------------------------------------------------------------
  */
 static int
@@ -424,14 +412,14 @@ TclX_ScanmatchObjCmd (clientData, interp, objc, objv)
 {
     scanContext_t  *contextPtr, **tableEntryPtr;
     matchDef_t     *newmatch;
-    int             compFlags = TCLX_REXP_BOTH_ALGORITHMS;
+    int             regExpFlags = REG_ADVANCED;
     int             firstArg = 1;
 
     if (objc < 3)
         goto argError;
 
     if (STREQU (Tcl_GetStringFromObj (objv[1], NULL), "-nocase")) {
-        compFlags |= TCLX_REXP_NO_CASE;
+        regExpFlags |= REG_ICASE;
         firstArg = 2;
     }
       
@@ -456,9 +444,9 @@ TclX_ScanmatchObjCmd (clientData, interp, objc, objv)
     if (objc == 3) {
         if (contextPtr->defaultAction) {
             Tcl_AppendStringsToObj (Tcl_GetObjResult (interp),
-		Tcl_GetStringFromObj (objv[0], NULL),
-		": default match already specified in this scan context", 
-		(char *) NULL);
+                                    Tcl_GetStringFromObj (objv[0], NULL),
+                                    ": default match already specified in this scan context", 
+                                    (char *) NULL);
             return TCL_ERROR;
         }
 	Tcl_IncrRefCount (objv [2]);
@@ -472,21 +460,18 @@ TclX_ScanmatchObjCmd (clientData, interp, objc, objv)
      */
 
     newmatch = (matchDef_t *) ckalloc(sizeof (matchDef_t));
-    newmatch->matchflags = 0;
 
-    if (compFlags & TCLX_REXP_NO_CASE) {
-        newmatch->matchflags |= MATCH_CASE_INSENSITIVE_FLAG;
-        contextPtr->flags |= CONTEXT_A_CASE_INSENSITIVE_FLAG;
-    }
-
-    if (TclX_RegExpCompileObj (interp, &newmatch->regExpInfo,
-              objv [firstArg + 1], compFlags) != TCL_OK) {
+    newmatch->regExp = (Tcl_RegExp)
+        Tcl_GetRegExpFromObj(interp, objv[firstArg + 1], regExpFlags);
+    if (newmatch->regExp == NULL) {
         ckfree ((char *) newmatch);
-        return (TCL_ERROR);
+	return TCL_ERROR;
     }
 
-    Tcl_IncrRefCount (objv [firstArg + 2]);
+    newmatch->regExpObj = objv[firstArg + 1],
+    Tcl_IncrRefCount (newmatch->regExpObj);
     newmatch->command = objv [firstArg + 2];
+    Tcl_IncrRefCount (newmatch->command);
 
     /*
      * Link in the new match.
@@ -513,10 +498,10 @@ argError:
  * only once.
  *
  * Parameters:
- *   o interp (O) - The Tcl interpreter to set the matchInfo variable in.
+ *   o interp - The Tcl interpreter to set the matchInfo variable in.
  *     Errors are returned in result.
- *   o scanData (I/O) - Data about the current line being scanned.
- *     been stored.  Should be set to FALSE when a new line is read.
+ *   o scanData - Data about the current line being scanned.
+ *     been stored.
  *-----------------------------------------------------------------------------
  */
 static int
@@ -526,8 +511,13 @@ SetMatchInfoVar (interp, scanData)
 {
     static char *MATCHINFO = "matchInfo";
     int idx, start, end;
+    char *value;
+    Tcl_DString valueBuf;
     char key [32];
     Tcl_Obj *valueObjPtr, *indexObjv [2];
+    TclRegexp *regExpPtr;
+
+    Tcl_DStringInit(&valueBuf);
 
     /*
      * Save information about the current line, if it hasn't been saved.
@@ -539,13 +529,13 @@ SetMatchInfoVar (interp, scanData)
         
         if (Tcl_SetVar2 (interp, MATCHINFO, "line", scanData->line, 
                          TCL_LEAVE_ERR_MSG) == NULL)
-            return TCL_ERROR;
+            goto errorExit;
 
         valueObjPtr = Tcl_NewLongObj ((long) scanData->offset);
-        if (TclX_ObjSetVar2S (interp, MATCHINFO, "offset", valueObjPtr,
-                              TCL_LEAVE_ERR_MSG) == NULL) {
+        if (Tcl_SetVar2Ex(interp, MATCHINFO, "offset", valueObjPtr,
+                          TCL_LEAVE_ERR_MSG) == NULL) {
             Tcl_DecrRefCount (valueObjPtr);
-            return TCL_ERROR;
+            goto errorExit;
         }
 
 #if 0
@@ -555,69 +545,81 @@ SetMatchInfoVar (interp, scanData)
          * disabled.
          */
         valueObjPtr = Tcl_NewLongObj ((long) scanData->bytesRead);
-        if (TclX_SetVar2S (interp, MATCHINFO, "bytesread", valueObjPtr,
-                           TCL_LEAVE_ERR_MSG) == NULL) {
+        if (Tcl_SetObjVar2 (interp, MATCHINFO, "bytesread", valueObjPtr,
+                            TCL_LEAVE_ERR_MSG) == NULL) {
             Tcl_DecrRefCount (valueObjPtr);
-            return TCL_ERROR;
+            goto errorExit;
         }
 #endif
         valueObjPtr = Tcl_NewIntObj ((long) scanData->lineNum);
-        if (TclX_ObjSetVar2S (interp, MATCHINFO, "linenum", valueObjPtr,
-                              TCL_LEAVE_ERR_MSG) == NULL) {
+        if (Tcl_SetVar2Ex(interp, MATCHINFO, "linenum", valueObjPtr,
+                          TCL_LEAVE_ERR_MSG) == NULL) {
             Tcl_DecrRefCount (valueObjPtr);
-            return TCL_ERROR;
+            goto errorExit;
         }
 
         if (Tcl_SetVar2 (interp, MATCHINFO, "context",
                          scanData->contextPtr->contextHandle,
                          TCL_LEAVE_ERR_MSG) == NULL)
-            return TCL_ERROR;
+            goto errorExit;
 
         if (Tcl_SetVar2 (interp, MATCHINFO, "handle", 
                          Tcl_GetChannelName (scanData->channel),
                          TCL_LEAVE_ERR_MSG) == NULL)
-            return TCL_ERROR;
+            goto errorExit;
 
     }
 
     if (scanData->contextPtr->copyFileChannel != NULL) {
         if (Tcl_SetVar2 (interp, MATCHINFO, "copyHandle", 
-                Tcl_GetChannelName (scanData->contextPtr->copyFileChannel),
-                TCL_LEAVE_ERR_MSG) == NULL)
-            return TCL_ERROR;
+                         Tcl_GetChannelName (scanData->contextPtr->copyFileChannel),
+                         TCL_LEAVE_ERR_MSG) == NULL)
+            goto errorExit;
     }
 
-    if (scanData->matchPtr == NULL)
-        return TCL_OK;
+    if (scanData->matchPtr == NULL) {
+        goto exitPoint;
+    }
+    regExpPtr = (TclRegexp *) scanData->matchPtr->regExp;
 
-    for (idx = 0; idx < scanData->matchPtr->regExpInfo.numSubExprs; idx++) {
-        start = scanData->subMatchInfo [idx].start;
-        end = scanData->subMatchInfo [idx].end;
+    for (idx = 0; idx < regExpPtr->re.re_nsub; idx++) {
+	start = regExpPtr->matches[idx+1].rm_so;
+	end = regExpPtr->matches[idx+1].rm_eo;
 
         sprintf (key, "subindex%d", idx);
         indexObjv [0] = Tcl_NewIntObj (start);
-        indexObjv [1] = Tcl_NewIntObj (end);
+        if (start < 0) {
+            indexObjv [1] = Tcl_NewIntObj (-1);
+        } else {
+            indexObjv [1] = Tcl_NewIntObj (end-1);
+        }
         valueObjPtr = Tcl_NewListObj (2, indexObjv);
-        if (TclX_ObjSetVar2S (interp, "matchInfo", key, valueObjPtr,
-                              TCL_LEAVE_ERR_MSG) == NULL) {
+        if (Tcl_SetVar2Ex(interp, MATCHINFO, key, valueObjPtr,
+                            TCL_LEAVE_ERR_MSG) == NULL) {
             Tcl_DecrRefCount (valueObjPtr);
-            return TCL_ERROR;
+            goto errorExit;
         }
 
         sprintf (key, "submatch%d", idx);
-        if (start < 0) {
-            valueObjPtr = Tcl_NewStringObj ("", 0);
-        } else {
-            valueObjPtr = Tcl_NewStringObj (scanData->line + start,
-                                            (end - start) + 1);
-        }
-        if (TclX_ObjSetVar2S (interp, "matchInfo", key, valueObjPtr,
-                              TCL_LEAVE_ERR_MSG) == NULL) {
+        Tcl_DStringSetLength(&valueBuf, 0);
+        value = Tcl_UniCharToUtfDString(scanData->uniLine + start, end - start,
+                                        &valueBuf);
+        valueObjPtr = Tcl_NewStringObj(value, (end - start));
+
+        if (Tcl_SetVar2Ex(interp, MATCHINFO, key, valueObjPtr,
+                            TCL_LEAVE_ERR_MSG) == NULL) {
             Tcl_DecrRefCount (valueObjPtr);
-            return TCL_ERROR;
+            goto errorExit;
         }
     }
+
+  exitPoint:
+    Tcl_DStringFree(&valueBuf);
     return TCL_OK;
+
+  errorExit:
+    Tcl_DStringFree(&valueBuf);
+    return TCL_ERROR;
 }
 
 /*-----------------------------------------------------------------------------
@@ -632,9 +634,10 @@ ScanFile (interp, contextPtr, channel)
     scanContext_t *contextPtr;
     Tcl_Channel    channel;
 {
-    Tcl_DString buffer, lowerBuffer;
+    Tcl_DString lineBuf, uniLineBuf;
     int result, matchedAtLeastOne;
     scanData_t data;
+    int matchStat;
     
     if (contextPtr->matchListHead == NULL) {
         TclX_AppendObjResult (interp, "no patterns in current scan context",
@@ -648,8 +651,8 @@ ScanFile (interp, contextPtr, channel)
     data.bytesRead = 0;
     data.lineNum = 0;
     
-    Tcl_DStringInit (&buffer);
-    Tcl_DStringInit (&lowerBuffer);
+    Tcl_DStringInit (&lineBuf);
+    Tcl_DStringInit (&uniLineBuf);
 
     result = TCL_OK;
     while (TRUE) {
@@ -657,8 +660,8 @@ ScanFile (interp, contextPtr, channel)
             goto scanExit;  /* Closed by a callback */
 
         data.offset = Tcl_Tell (channel);
-        Tcl_DStringSetLength (&buffer, 0);
-        if (Tcl_Gets (channel, &buffer) < 0) {
+        Tcl_DStringSetLength (&lineBuf, 0);
+        if (Tcl_Gets (channel, &lineBuf) < 0) {
             if (Tcl_Eof (channel) || Tcl_InputBlocked (channel))
                 goto scanExit;
             Tcl_SetStringObj (Tcl_GetObjResult (interp),
@@ -666,33 +669,38 @@ ScanFile (interp, contextPtr, channel)
             result = TCL_ERROR;
             goto scanExit;
         }
-        data.line = buffer.string;
-        data.bytesRead += (buffer.length + 1);  /* Include EOLN */
+
+
+        data.line = Tcl_DStringValue(&lineBuf);
+        data.bytesRead += (lineBuf.length + 1);  /* Include EOLN */
         data.lineNum++;
         data.storedLine = FALSE;
-        matchedAtLeastOne = FALSE;
 
-        if (contextPtr->flags & CONTEXT_A_CASE_INSENSITIVE_FLAG) {
-            Tcl_DStringSetLength (&lowerBuffer, 0);
-            Tcl_DStringAppend (&lowerBuffer, buffer.string, -1);
-            TclX_DownShift (lowerBuffer.string, lowerBuffer.string);
-        }
+        /* Convert to UTF to UniCode */
+        Tcl_DStringSetLength (&uniLineBuf, 0);
+        data.uniLine = Tcl_UtfToUniCharDString(Tcl_DStringValue(&lineBuf),
+                                               Tcl_DStringLength(&lineBuf),
+                                               &uniLineBuf);
+        data.uniLineLen = Tcl_DStringLength(&uniLineBuf) / sizeof(Tcl_UniChar);
+
+        matchedAtLeastOne = FALSE;
 
         for (data.matchPtr = contextPtr->matchListHead; 
              data.matchPtr != NULL; 
              data.matchPtr = data.matchPtr->nextMatchDefPtr) {
 
-            if (!TclX_RegExpExecute (interp,
-                                     &(data.matchPtr->regExpInfo),
-                                     buffer.string,
-                                     lowerBuffer.string,
-                                     data.subMatchInfo))
+            matchStat = TclRegExpExecUniChar(interp, data.matchPtr->regExp,
+                                             data.uniLine, data.uniLineLen, -1, 0);
+            if (matchStat < 0) {
+                result = TCL_ERROR;
+                goto scanExit;
+            }
+            if (matchStat == 0) {
                 continue;  /* Try next match pattern */
-
+            }
             matchedAtLeastOne = TRUE;
 
-            result = SetMatchInfoVar (interp,
-                                      &data);
+            result = SetMatchInfoVar (interp, &data);
             if (result != TCL_OK)
                 goto scanExit;
 
@@ -744,8 +752,8 @@ ScanFile (interp, contextPtr, channel)
         }
 
 	if ((contextPtr->copyFileChannel != NULL) && (!matchedAtLeastOne)) {
-	    if ((Tcl_Write (contextPtr->copyFileChannel,
-                            buffer.string, buffer.length) < 0) ||
+	    if ((Tcl_Write (contextPtr->copyFileChannel, Tcl_DStringValue(&lineBuf),
+                            Tcl_DStringLength(&lineBuf)) < 0) ||
                 (TclX_WriteNL (contextPtr->copyFileChannel) < 0)) {
                 Tcl_SetStringObj (Tcl_GetObjResult (interp),
                                   Tcl_PosixError (interp), -1);
@@ -755,8 +763,8 @@ ScanFile (interp, contextPtr, channel)
     }
 
   scanExit:
-    Tcl_DStringFree (&buffer);
-    Tcl_DStringFree (&lowerBuffer);
+    Tcl_DStringFree (&lineBuf);
+    Tcl_DStringFree (&uniLineBuf);
     if (result == TCL_ERROR)
         return TCL_ERROR;
     return TCL_OK;
