@@ -3,7 +3,7 @@
  *
  * A collection of general commands: echo, infox and loop.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1997 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXgeneral.c,v 8.11.2.1 1998/09/28 19:01:28 markd Exp $
+ * $Id$
  *-----------------------------------------------------------------------------
  */
 
@@ -47,6 +47,11 @@ TclX_LoopObjCmd _ANSI_ARGS_((ClientData clientData,
                              int         objc,
                              Tcl_Obj    *CONST objv[]));
 
+static int
+SetLoopCounter _ANSI_ARGS_((Tcl_Interp *interp,
+                            char *varName,
+                            int idx));
+
 static int
 GlobalImport _ANSI_ARGS_((Tcl_Interp *interp));
 
@@ -302,6 +307,40 @@ TclX_InfoxObjCmd (clientData, interp, objc, objv)
                           (char *) NULL);
     return TCL_ERROR;
 }
+
+
+/*-----------------------------------------------------------------------------
+ * SetLoopCounter --
+ *   Set the loop command counter variable.
+ *-----------------------------------------------------------------------------
+ */
+static int
+SetLoopCounter (interp, varName, idx)
+    Tcl_Interp *interp;
+    char *varName;
+    int idx;
+{
+    Tcl_Obj *iObj, *newVarObj;
+
+    iObj = Tcl_GetVar2Ex(interp, varName, NULL, TCL_PARSE_PART1);
+    if ((iObj == NULL) || (Tcl_IsShared (iObj))) {
+        iObj = newVarObj = Tcl_NewLongObj (idx);
+    } else {
+        newVarObj = NULL;
+    }
+
+    Tcl_SetLongObj (iObj, idx);
+    if (Tcl_SetVar2Ex(interp, varName, NULL, iObj,
+                      TCL_PARSE_PART1|TCL_LEAVE_ERR_MSG) == NULL) {
+        if (newVarObj != NULL) {
+            Tcl_DecrRefCount (newVarObj);
+        }
+        return TCL_ERROR;
+    }
+    return TCL_OK;
+}
+
+
 
 /*-----------------------------------------------------------------------------
  * TclX_LoopObjCmd --
@@ -319,9 +358,10 @@ TclX_LoopObjCmd (dummy, interp, objc, objv)
     int         objc;
     Tcl_Obj    *CONST objv[];
 {
-    int       result = TCL_OK;
-    long      i, first, limit, incr = 1;
-    Tcl_Obj  *command, *iObj, *newVarObj;
+    int result = TCL_OK;
+    long idx, first, limit, incr = 1;
+    char *varName; 
+    Tcl_Obj  *command;
 
     if ((objc < 5) || (objc > 6)) {
         return TclX_WrongArgs (interp, objv [0], 
@@ -342,29 +382,13 @@ TclX_LoopObjCmd (dummy, interp, objc, objv)
         command = objv [5];
     }
 
-    for (i = first;
-         (((i < limit) && (incr >= 0)) || ((i > limit) && (incr < 0)));
-         i += incr) {
+    varName = Tcl_GetStringFromObj (objv[1], NULL);
+    for (idx = first;
+         (((idx < limit) && (incr >= 0)) || ((idx > limit) && (incr < 0)));
+         idx += incr) {
         
-        /*
-         * Set loop counter.  The newVarObj var is used to determine if object should be
-         * deleted on error.
-         */
-        iObj = Tcl_ObjGetVar2 (interp, objv [1], (Tcl_Obj *) NULL,
-                               TCL_PARSE_PART1);
-        if ((iObj == NULL) || (Tcl_IsShared (iObj))) {
-            iObj = newVarObj = Tcl_NewLongObj (first);
-        } else {
-            newVarObj = NULL;
-        }
-        Tcl_SetLongObj (iObj, i);
-        if (Tcl_ObjSetVar2 (interp, objv [1], (Tcl_Obj *) NULL, iObj,
-                            TCL_PARSE_PART1 | TCL_LEAVE_ERR_MSG) == NULL) {
-            if (newVarObj != NULL) {
-                Tcl_DecrRefCount (newVarObj);
-            }
+        if (SetLoopCounter(interp, varName, idx) == TCL_ERROR)
             return TCL_ERROR;
-        }
 
         result = Tcl_EvalObj (interp, command);
         if (result == TCL_CONTINUE) {
@@ -387,23 +411,8 @@ TclX_LoopObjCmd (dummy, interp, objc, objv)
     /*
      * Set loop counter to its final value.
      */
-    iObj = Tcl_ObjGetVar2 (interp, objv [1], (Tcl_Obj *) NULL,
-                           TCL_PARSE_PART1);
-    if ((iObj == NULL) || (Tcl_IsShared (iObj))) {
-        iObj = newVarObj = Tcl_NewLongObj (first);
-    } else {
-        newVarObj = NULL;
-    }
-
-    Tcl_SetLongObj (iObj, i);
-
-    if (Tcl_ObjSetVar2 (interp, objv [1], (Tcl_Obj *) NULL, iObj,
-                        TCL_PARSE_PART1 | TCL_LEAVE_ERR_MSG) == NULL) {
-        if (newVarObj != NULL) {
-            Tcl_DecrRefCount (newVarObj);
-        }
+    if (SetLoopCounter(interp, varName, idx) == TCL_ERROR)
         return TCL_ERROR;
-    }
     return result;
 }
 
@@ -511,8 +520,8 @@ TclX_Try_EvalObjCmd (dummy, interp, objc, objv)
 
         code = GlobalImport (interp);
         if (code != TCL_ERROR) {
-            if (TclX_ObjSetVar2S (interp, "errorResult", NULL, 
-                                  resultObjPtr, TCL_LEAVE_ERR_MSG) == NULL) {
+            if (Tcl_SetVar2Ex(interp, "errorResult", NULL, 
+                              resultObjPtr, TCL_LEAVE_ERR_MSG) == NULL) {
                 code = TCL_ERROR;
             }
         }
