@@ -3,7 +3,7 @@
  *
  * Extended Tcl initialzation and initialization utilitied.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1997 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXinit.c,v 8.12.2.1 1998/09/22 02:52:51 markd Exp $
+ * $Id$
  *-----------------------------------------------------------------------------
  */
 
@@ -40,28 +40,37 @@
  * Globals:
  *   o ${w}x_library - Set to the directory containing the init file.
  */
+/*FIX: Break into 3 parts and don't do source if standalone isn't
+ * available*/
 static char tclx_findinit [] =
 "proc tclx_findinit {w defaultLib version noInit} {\n\
     upvar #0 env env ${w}x_library libDir tcl_platform tcl_platform\n\
     set dirs {}\n\
     set envVar [string toupper ${w}X_LIBRARY]\n\
-    if [info exists env($envVar)] {lappend dirs $env($envVar)}\n\
-    if [info exists env(EXT_FOLDER)] {\n\
-	lappend dirs [file join $env(EXT_FOLDER) \"Tool Command Language\" ${w}X$version]\n\
+    if {[info exists libDir]} {\n\
+        lappend dirs $libDir\n\
+    } else {\n\
+        if [info exists env($envVar)] {lappend dirs $env($envVar)}\n\
+        lappend dirs [file join [file dirname [info library]] ${w}X$version]\n\
+        if [info exists env(EXT_FOLDER)] {\n\
+	    lappend dirs [file join $env(EXT_FOLDER) \"Tool Command Language\" ${w}X$version]\n\
+        }\n\
+	if {[string length $defaultLib]} {\n\
+            lappend dirs $defaultLib\n\
+	}\n\
+        set libDir {}\n\
+        if ![catch {uplevel #0 source -rsrc ${w}x}] {\n\
+	    uplevel #0 source -rsrc ${w}x:tclIndex\n\
+	    return\n\
+        }\n\
+        set prefix [file dirname [info nameofexecutable]]\n\
+        set plat [file tail $prefix]\n\
+        set prefix [file dirname $prefix]\n\
+        lappend dirs [file join $prefix lib ${w}X$version]\n\
+        set prefix [file dirname $prefix]\n\
+        lappend dirs [file join $prefix ${w}X${version} $w $plat]\n\
+        lappend dirs [file join [file dirname $prefix] ${w}X${version} $w $plat]\n\
     }\n\
-    lappend dirs $defaultLib\n\
-    set libDir {}\n\
-    if ![catch {uplevel #0 source -rsrc ${w}x}] {\n\
-	uplevel #0 source -rsrc ${w}x:tclIndex\n\
-	return\n\
-    }\n\
-    set prefix [file dirname [info nameofexecutable]]\n\
-    set plat [file tail $prefix]\n\
-    set prefix [file dirname $prefix]\n\
-    lappend dirs [file join $prefix lib ${w}X$version]\n\
-    set prefix [file dirname $prefix]\n\
-    lappend dirs [file join $prefix ${w}X${version} $w $plat]\n\
-    lappend dirs [file join [file dirname $prefix] ${w}X${version} $w $plat]\n\
     foreach libDir $dirs {\n\
         set init [file join $libDir ${w}x.tcl]\n\
         if [file exists $init] {\n\
@@ -116,6 +125,7 @@ TclXRuntimeInit (interp, which, defaultLib, version)
     char       *version;
     
 {
+    /*FIX: Use objects */
 #define PROC_ARGC 5
     Tcl_CmdInfo cmdInfo;
     char *procArgv [PROC_ARGC + 1], *quick;
