@@ -3,7 +3,7 @@
  *
  * Tcl commands to load libraries of Tcl code.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1997 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXlib.c,v 8.19 1997/11/11 05:33:16 markd Exp $
+ * $Id$
  *-----------------------------------------------------------------------------
  */
 
@@ -37,8 +37,21 @@
  */
 static char *AUTO_INDEX     = "auto_index";
 static char *AUTO_PKG_INDEX = "auto_pkg_index";
-static char *TCLX_LIBRARY   = "tclx_library";
-static char *AUTO_LOAD_FILE = "autoload.tcl";
+
+/*
+ * Command to pass to Tcl_GlobalEval to load the file autoload.tcl.
+ * This is a global rather than a local so it will work with K&R compilers.
+ * Its writable so it works with gcc.
+ */
+#ifdef HAVE_TCL_STANDALONE
+static char autoloadCmd [] =
+"if [catch {source -rsrc autoload}] {\n\
+    source [file join $tclx_library autoload.tcl]\n\
+}";
+#else
+static char autoloadCmd [] =
+    "source [file join $tclx_library autoload.tcl]";
+#endif
 
 /*
  * Indicates the type of library index.
@@ -261,9 +274,10 @@ MakeAbsFile (interp, fileName, absNamePtr)
     Tcl_DString *absNamePtr;
 {
     char  *curDir;
-    Tcl_DString joinBuf;
+    Tcl_DString joinBuf, cwdBuffer;
 
     Tcl_DStringSetLength (absNamePtr, 1);
+    Tcl_DStringInit (&cwdBuffer);
 
     fileName = Tcl_TranslateFileName (interp, fileName, absNamePtr);
     if (fileName == NULL)
@@ -284,7 +298,7 @@ MakeAbsFile (interp, fileName, absNamePtr)
      * Otherwise its relative to the current directory, get the directory
      * and join into a path.
      */
-    curDir = TclGetCwd (interp);
+    curDir = TclpGetCwd (interp, &cwdBuffer);
     if (curDir == NULL)
         goto errorExit;
 
@@ -294,9 +308,11 @@ MakeAbsFile (interp, fileName, absNamePtr)
     Tcl_DStringAppend (absNamePtr, joinBuf.string, -1);
     Tcl_DStringFree (&joinBuf);
 
+    Tcl_DStringFree (&cwdBuffer);
     return Tcl_DStringValue (absNamePtr);
 
   errorExit:
+    Tcl_DStringFree (&cwdBuffer);
     return NULL;
 }
 
@@ -337,8 +353,8 @@ SetPackageIndexEntry (interp, packageName, fileName, offset, length)
     pkgDataObjv [2] = Tcl_NewIntObj ((int) length);
     pkgDataPtr = Tcl_NewListObj (3, pkgDataObjv);
 
-    if (TclX_ObjSetVar2S (interp, AUTO_PKG_INDEX, packageName, pkgDataPtr,
-                          TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG) == NULL) {
+    if (Tcl_SetVar2Ex(interp, AUTO_PKG_INDEX, packageName, pkgDataPtr,
+                      TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG) == NULL) {
         Tcl_DecrRefCount (pkgDataPtr);
         return TCL_ERROR;
     }
@@ -376,8 +392,8 @@ GetPackageIndexEntry (interp, packageName, fileNamePtr, offsetPtr, lengthPtr)
     /*
      * Look up the package entry in the array.
      */
-    pkgDataPtr = TclX_ObjGetVar2S (interp, AUTO_PKG_INDEX, packageName,
-                                   TCL_GLOBAL_ONLY);
+    pkgDataPtr = Tcl_GetVar2Ex(interp, AUTO_PKG_INDEX, packageName,
+                               TCL_GLOBAL_ONLY);
     if (pkgDataPtr == NULL) {
         TclX_AppendObjResult (interp, "entry not found in \"auto_pkg_index\"",
                               " for package \"", packageName, "\"",
@@ -958,25 +974,10 @@ int
 TclX_LibraryInit (interp)
     Tcl_Interp *interp;
 {
-    Tcl_Obj *libDirObj;
-    Tcl_DString autoLoadSrc;
     int result;
 
-    /*
-     * Get path to autoload.tcl file that contains modified version of 
-     * the Tcl autoload proc and eval the file.
-     */
-    libDirObj = TclX_ObjGetVar2S (interp, TCLX_LIBRARY, NULL, 
-                                  TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG);
-    if (libDirObj == NULL) {
-        return TCL_ERROR;
-    }
-    Tcl_DStringInit (&autoLoadSrc);
-    TclX_JoinPath (Tcl_GetStringFromObj (libDirObj, NULL),
-                   AUTO_LOAD_FILE, &autoLoadSrc);
-    result = TclX_Eval (interp, TCLX_EVAL_GLOBAL|TCLX_EVAL_FILE,
-                        autoLoadSrc.string);
-    Tcl_DStringFree (&autoLoadSrc);
+    /* Hack in our own auto-loading */
+    result = TclX_Eval (interp, TCLX_EVAL_GLOBAL, autoloadCmd);
     if (result == TCL_ERROR) {
         return TCL_ERROR;
     }
