@@ -3,7 +3,7 @@
  *
  * Tcl command to create and manage processes.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1997 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXprocess.c,v 8.8 1997/07/04 20:23:58 markd Exp $
+ * $Id$
  *-----------------------------------------------------------------------------
  */
 
@@ -237,7 +237,7 @@ TclX_WaitObjCmd (clientData, interp, objc, objv)
             pid = 0;
     }
 
-    returnedPid = TCLX_WAITPID (pid, (int *) (&status), options);
+    returnedPid = (pid_t) TCLX_WAITPID (pid, (int *) (&status), options);
 
     if (returnedPid < 0) {
         TclX_AppendObjResult (interp, "wait for process failed: ",
