@@ -1,9 +1,15 @@
 /*
  * tclXselect.c
  *
- * Extended Tcl file I/O commands.
+ * Select command.  This is the generic code associated with the select system
+ * call.  It relies on the Unix style select, which operates on bit sets of
+ * file numbers.  Platform specific code is called to translate channels into
+ * file numbers, but all operations are generic.  On Win32, this only works
+ * on sockets.  Ideally, it would push more code into the platform specific
+ * modules and work on more file types.  However, right now, I don't see a
+ * good way to do this on Win32.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +18,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXselect.c,v 7.0 1996/06/16 05:30:50 markd Exp $
+ * $Id$
  *-----------------------------------------------------------------------------
  */
 
@@ -24,25 +30,6 @@
 #   include <sys/select.h>
 #endif
 
-/*
- * Cheat a little to avoid configure checking for floor being prototyped.
- * This breaks with GNU libc headers...really should check with autoconf.
- */
-
-#ifndef __GNU_LIBRARY__
-extern
-double floor ();
-#endif
-
-/*
- * A couple of systems (Xenix and older SCO unix) have bzero hidden away
- * in the X library that we don't use, but the select macros use bzero.
- * Make them use memset with this magic.
- */
-#ifndef NO_BZERO
-#    define bzero(to,length)    memset(to,'\0',length)
-#endif
-
 /*
  * A few systems (A/UX 2.0) have select but no macros, define em in this case.
  */
@@ -57,9 +44,10 @@ double floor ();
  * Data kept about a file channel.
  */
 typedef struct {
-    Tcl_Channel channel;
-    int         readFd;
-    int         writeFd;
+    Tcl_Obj     *channelIdObj;
+    Tcl_Channel  channel;
+    int          readFd;
+    int          writeFd;
 } channelData_t;
 
 /*
@@ -67,7 +55,8 @@ typedef struct {
  */
 static int
 ParseSelectFileList _ANSI_ARGS_((Tcl_Interp     *interp,
-                                 char           *handleList,
+                                 int             chanAccess,
+                                 Tcl_Obj        *handleList,
                                  fd_set         *fileSetPtr,
                                  channelData_t **channelListPtr,
                                  int            *maxFileIdPtr));
@@ -77,12 +66,17 @@ FindPendingData _ANSI_ARGS_((int            fileDescCnt,
                              channelData_t *channelList,
                              fd_set        *fileDescSetPtr));
 
-static char *
+static Tcl_Obj *
 ReturnSelectedFileList _ANSI_ARGS_((fd_set        *fileDescSetPtr,
-                                    fd_set        *fileDescSet2Ptr,
                                     int            fileDescCnt,
                                     channelData_t *channelListPtr));
 
+static int 
+TclX_SelectObjCmd _ANSI_ARGS_((ClientData clientData, 
+                               Tcl_Interp *interp,
+                               int objc,
+                               Tcl_Obj *CONST objv[]));
+
 
 /*-----------------------------------------------------------------------------
  * ParseSelectFileList --
@@ -90,49 +84,55 @@ ReturnSelectedFileList _ANSI_ARGS_((fd_set        *fileDescSetPtr,
  *   Parse a list of file handles for select.
  *
  * Parameters:
- *   o interp (O) - Error messages are returned in the result.
+ *   o interp - Error messages are returned in the result.
+ *   o chanAccess - TCL_READABLE for read direction, TCL_WRITABLE for write
+ *     direction or both for both files.
  *   o handleList (I) - The list of file handles to parse, may be empty.
- *   o fileSetPtr (O) - The select fd_set for the parsed handles is
+ *   o fileSetPtr - The select fd_set for the parsed handles is
  *     filled in.
- *   o channelListPtr (O) - A pointer to a dynamically allocated list of
+ *   o channelListPtr - A pointer to a dynamically allocated list of
  *     the channels that are in the set.  If the list is empty, NULL is
  *     returned.
  *   o maxFileIdPtr (I/O) - If a file id greater than the current value is
  *     encountered, it will be set to that file id.
  * Returns:
  *   The number of files in the list, or -1 if an error occured.
+ * FIX: Should really pass in access and only get channels that are 
+ * requested.
  *-----------------------------------------------------------------------------
  */
 static int
-ParseSelectFileList (interp, handleList, fileSetPtr, channelListPtr,
-                     maxFileIdPtr)
+ParseSelectFileList (interp, chanAccess, handleList, fileSetPtr,
+                     channelListPtr, maxFileIdPtr)
     Tcl_Interp    *interp;
-    char          *handleList;
+    int            chanAccess;
+    Tcl_Obj       *handleList;
     fd_set        *fileSetPtr;
     channelData_t **channelListPtr;
     int           *maxFileIdPtr;
 {
     int handleCnt, idx;
-    char **handleArgv;
+    Tcl_Obj **handleObjv;
     channelData_t *channelList;
 
     /*
      * Optimize empty list handling.
      */
-    if (handleList [0] == '\0') {
+    if (TclX_IsNullObj (handleList)) {
         *channelListPtr = NULL;
         return 0;
     }
 
-    if (Tcl_SplitList (interp, handleList, &handleCnt, &handleArgv) != TCL_OK)
+    if (Tcl_ListObjGetElements (interp, handleList, &handleCnt,
+                                &handleObjv) != TCL_OK) {
         return -1;
+    }
 
     /*
      * Handle case of an empty list.
      */
     if (handleCnt == 0) {
         *channelListPtr = NULL;
-        ckfree ((char *) handleArgv);
         return 0;
     }
 
@@ -140,35 +140,48 @@ ParseSelectFileList (interp, handleList, fileSetPtr, channelListPtr,
         (channelData_t*) ckalloc (sizeof (channelData_t) * handleCnt);
 
     for (idx = 0; idx < handleCnt; idx++) {
+        channelList [idx].channelIdObj = handleObjv [idx];
         channelList [idx].channel =
-            TclX_GetOpenChannel (interp,
-                                 handleArgv [idx],
-                                 0);
-        if (channelList [idx].channel == NULL) {
-            ckfree ((char *) handleArgv);
-            ckfree ((char *) channelList);
-            return -1;
-        }
-        
-        channelList [idx].readFd =
-            TclX_ChannelFnum (channelList [idx].channel, TCL_READABLE);
-        if (channelList [idx].readFd >= 0) {
+            TclX_GetOpenChannelObj (interp,
+                                    handleObjv [idx],
+                                    chanAccess);
+        if (channelList [idx].channel == NULL)
+            goto errorExit;
+
+        if (chanAccess & TCL_READABLE) {
+            if (TclXOSGetSelectFnum (interp, 
+                                     channelList [idx].channel,
+                                     TCL_READABLE,
+                                     &channelList [idx].readFd) != TCL_OK)
+                goto errorExit;
             FD_SET (channelList [idx].readFd, fileSetPtr);
             if (channelList [idx].readFd > *maxFileIdPtr)
                 *maxFileIdPtr = channelList [idx].readFd;
+        } else {
+            channelList [idx].readFd = -1;
         }
-        channelList [idx].writeFd =
-            TclX_ChannelFnum (channelList [idx].channel, TCL_WRITABLE);
-        if (channelList [idx].writeFd >= 0) {
+
+        if (chanAccess & TCL_WRITABLE) {
+            if (TclXOSGetSelectFnum (interp, 
+                                     channelList [idx].channel,
+                                     TCL_WRITABLE,
+                                     &channelList [idx].writeFd) != TCL_OK)
+                goto errorExit;
             FD_SET (channelList [idx].writeFd, fileSetPtr);
             if (channelList [idx].writeFd > *maxFileIdPtr)
                 *maxFileIdPtr = channelList [idx].writeFd;
+        } else {
+            channelList [idx].writeFd = -1;
         }
     }
 
     *channelListPtr = channelList;
-    ckfree ((char *) handleArgv);
     return handleCnt;
+
+  errorExit:
+    ckfree ((char *) channelList);
+    return -1;
+
 }
 
 /*-----------------------------------------------------------------------------
@@ -214,38 +227,21 @@ FindPendingData (fileDescCnt, channelList, fileDescSetPtr)
  *
  * Parameters:
  *   o fileDescSetPtr (I) - The select fd_set.
- *   o fileDescSet2Ptr (I) - Pointer to a second descriptor to also check
- *     (their may be overlap).  NULL if no second set.
  *   o fileDescCnt (I) - Number of descriptors in the list.
  *   o channelListPtr (I) - A pointer to a list of the FILE pointers for
  *     files that are in the set.
  * Returns:
- *   A dynamicly allocated list of file handles.  If the handles are empty,
- *   it returns a constant empty string.  Do not free the empty string.
+ *   List of file handles.
  *-----------------------------------------------------------------------------
  */
-static char *
-ReturnSelectedFileList (fileDescSetPtr, fileDescSet2Ptr, fileDescCnt,
-                        channelList) 
+static Tcl_Obj *
+ReturnSelectedFileList (fileDescSetPtr, fileDescCnt, channelList) 
     fd_set        *fileDescSetPtr;
-    fd_set        *fileDescSet2Ptr;
     int            fileDescCnt;
     channelData_t *channelList;
 {
     int idx, handleCnt;
-    char *fileHandleList;
-    char **fileHandleArgv;
-
-    /*
-     * Special case the empty list.
-     */
-    if (fileDescCnt == 0)
-        return "";
-
-    /*
-     * Allocate enough room to hold the argv.
-     */
-    fileHandleArgv = (char **) ckalloc (fileDescCnt * sizeof (char *));
+    Tcl_Obj *fileHandleList = Tcl_NewListObj (0, NULL);
 
     handleCnt = 0;
     for (idx = 0; idx < fileDescCnt; idx++) {
@@ -253,28 +249,17 @@ ReturnSelectedFileList (fileDescSetPtr, fileDescSet2Ptr, fileDescCnt,
              FD_ISSET (channelList [idx].readFd, fileDescSetPtr)) ||
             ((channelList [idx].writeFd >= 0) &&
              FD_ISSET (channelList [idx].writeFd, fileDescSetPtr))) {
-            fileHandleArgv [handleCnt] =
-                Tcl_GetChannelName (channelList [idx].channel);
-            handleCnt++;
-        } else if ((fileDescSet2Ptr != NULL) &&
-                   (((channelList [idx].readFd >= 0) &&
-                    FD_ISSET (channelList [idx].readFd, fileDescSet2Ptr)) ||
-                   ((channelList [idx].writeFd >= 0) &&
-                    FD_ISSET (channelList [idx].writeFd, fileDescSet2Ptr)))) {
-            fileHandleArgv [handleCnt] =
-                Tcl_GetChannelName (channelList [idx].channel);
+            Tcl_ListObjAppendElement (NULL, fileHandleList,
+                                      channelList [idx].channelIdObj);
             handleCnt++;
         }
     }
 
-    fileHandleList = Tcl_Merge (handleCnt, fileHandleArgv);
-    ckfree ((char *) fileHandleArgv);
-
     return fileHandleList;
 }
 
 /*-----------------------------------------------------------------------------
- * Tcl_SelectCmd --
+ * TclX_SelectObjCmd --
  *  Implements the select TCL command:
  *      select readhandles ?writehandles? ?excepthandles? ?timeout?
  *
@@ -287,29 +272,27 @@ ReturnSelectedFileList (fileDescSetPtr, fileDescSet2Ptr, fileDescCnt,
  *     or {} it the timeout expired.
  *-----------------------------------------------------------------------------
  */
-int
-Tcl_SelectCmd (clientData, interp, argc, argv)
-    ClientData  clientData;
-    Tcl_Interp *interp;
-    int         argc;
-    char      **argv;
+static int
+TclX_SelectObjCmd (clientData, interp, objc, objv)
+    ClientData   clientData;
+    Tcl_Interp  *interp;
+    int          objc;
+    Tcl_Obj     *CONST objv[];
 {
+    static int chanAccess [] = {TCL_READABLE, TCL_WRITABLE, 0};
     int idx;
     fd_set fdSets [3], readPendingFDSet;
     int descCnts [3];
     channelData_t *descLists [3];
-    char *retListArgv [3];
+    Tcl_Obj *handleSetList [3];
     int numSelected, maxFileId = 0, pending;
     int result = TCL_ERROR;
     struct timeval  timeoutRec;
     struct timeval *timeoutRecPtr;
 
-
-    if (argc < 2) {
-        Tcl_AppendResult (interp, tclXWrongArgs, argv [0], 
-                          " readFileIds ?writeFileIds? ?exceptFileIds?",
-                          " ?timeout?", (char *) NULL);
-        return TCL_ERROR;
+    if (objc < 2) {
+        return TclX_WrongArgs (interp, objv [0], 
+                      " readFileIds ?writeFileIds? ?exceptFileIds? ?timeout?");
     }
 
     /*
@@ -324,9 +307,12 @@ Tcl_SelectCmd (clientData, interp, argc, argv)
     /*
      * Parse the file handles and set everything up for the select call.
      */
-    for (idx = 0; (idx < 3) && (idx < argc - 1); idx++) {
-        descCnts [idx] = ParseSelectFileList (interp, argv [idx + 1],
-                                              &fdSets [idx], &descLists [idx],
+    for (idx = 0; (idx < 3) && (idx < objc - 1); idx++) {
+        descCnts [idx] = ParseSelectFileList (interp, 
+                                              chanAccess [idx],
+                                              objv [idx + 1],
+                                              &fdSets [idx],
+                                              &descLists [idx],
                                               &maxFileId);
         if (descCnts [idx] < 0)
             goto exitPoint;
@@ -336,20 +322,20 @@ Tcl_SelectCmd (clientData, interp, argc, argv)
      * Get the time out.  Zero is different that not specified.
      */
     timeoutRecPtr = NULL;
-    if ((argc > 4) && (argv [4][0] != '\0')) {
+    if ((objc > 4) && !TclX_IsNullObj (objv [4])) {
         double  timeout, seconds, microseconds;
 
-        if (Tcl_GetDouble (interp, argv [4], &timeout) != TCL_OK)
+        if (Tcl_GetDoubleFromObj (interp, objv [4], &timeout) != TCL_OK)
             goto exitPoint;
-        if (timeout < 0) {
-            Tcl_AppendResult (interp, "timeout must be greater than or equal",
-                              " to zero", (char *) NULL);
+        if (timeout < 0.0) {
+            TclX_AppendObjResult (interp, "timeout must be greater than ",
+                                  "or equal to zero", (char *) NULL);
             goto exitPoint;
         }
         seconds = floor (timeout);
         microseconds = (timeout - seconds) * 1000000.0;
-        timeoutRec.tv_sec = seconds;
-        timeoutRec.tv_usec = microseconds;
+        timeoutRec.tv_sec = (long) seconds;
+        timeoutRec.tv_usec = (long) microseconds;
         timeoutRecPtr = &timeoutRec;
     }
 
@@ -371,10 +357,21 @@ Tcl_SelectCmd (clientData, interp, argc, argv)
                           &fdSets [0], &fdSets [1], &fdSets [2],
                           timeoutRecPtr);
     if (numSelected < 0) {
-        Tcl_AppendResult (interp, "select error: ", Tcl_PosixError (interp),
-                          (char *) NULL);
+        TclX_AppendObjResult (interp, "select error: ",
+                              Tcl_PosixError (interp), (char *) NULL);
         goto exitPoint;
     }
+    
+    /*
+     * If there is read data pending in the buffers, force the bits to be set
+     * in the read fdSet.
+     */
+    if (pending) {
+        for (idx = 0; idx < descCnts [0]; idx++) {
+            if (FD_ISSET (descLists [0] [idx].readFd, &readPendingFDSet))
+                FD_SET (descLists [0] [idx].readFd, &(fdSets [0]));
+        }
+    }
 
     /*
      * Return the result, either a 3 element list, or leave the result
@@ -382,19 +379,12 @@ Tcl_SelectCmd (clientData, interp, argc, argv)
      */
     if (numSelected > 0 || pending) {
         for (idx = 0; idx < 3; idx++) {
-            retListArgv [idx] =
+            handleSetList [idx] =
                 ReturnSelectedFileList (&fdSets [idx],
-                                        (idx == 0) ? &readPendingFDSet : NULL,
                                         descCnts [idx],
                                         descLists [idx]);
         }
-
-        Tcl_SetResult (interp, Tcl_Merge (3, retListArgv), TCL_DYNAMIC); 
-        
-        for (idx = 0; idx < 3; idx++) {
-            if (retListArgv [idx][0] != '\0')
-                ckfree ((char *) retListArgv [idx]);
-        }
+        Tcl_SetObjResult (interp, Tcl_NewListObj (3, handleSetList)); 
     }
 
     result = TCL_OK;
@@ -408,21 +398,39 @@ Tcl_SelectCmd (clientData, interp, argc, argv)
 }
 #else
 /*-----------------------------------------------------------------------------
- * Tcl_SelectCmd --
+ * TclX_SelectCmd --
  *     Dummy select command that returns an error for systems that don't
  *     have select.
  *-----------------------------------------------------------------------------
  */
 int
-Tcl_SelectCmd (clientData, interp, argc, argv)
-    ClientData  clientData;
-    Tcl_Interp *interp;
-    int         argc;
-    char      **argv;
+TclX_SelectObjCmd (clientData, interp, objc, objv)
+    ClientData   clientData;
+    Tcl_Interp  *interp;
+    int          objc;
+    Tcl_Obj     *CONST objv[];
 {
-    Tcl_AppendResult (interp, 
-                      "select is not available on this version of Unix",
-                      (char *) NULL);
+    TclX_AppendObjResult (interp, 
+                          "select is not available on this version of Unix",
+                          (char *) NULL);
     return TCL_ERROR;
 }
 #endif
+
+
+/*-----------------------------------------------------------------------------
+ * TclX_SelectInit --
+ *     Initialize the select command.
+ *-----------------------------------------------------------------------------
+ */
+void
+TclX_SelectInit (interp)
+    Tcl_Interp *interp;
+{
+    Tcl_CreateObjCommand (interp, 
+                          "select",
+                          TclX_SelectObjCmd,
+                          (ClientData) NULL,
+                          (Tcl_CmdDeleteProc*) NULL);
+}
+
