@@ -3,7 +3,7 @@
 #
 # Proc to execute code on every line of a file.
 #------------------------------------------------------------------------------
-# Copyright 1992-1997 Karl Lehenbauer and Mark Diekhans.
+# Copyright 1992-1999 Karl Lehenbauer and Mark Diekhans.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose and without fee is hereby granted, provided
@@ -12,22 +12,25 @@
 # software for any purpose.  It is provided "as is" without express or
 # implied warranty.
 #------------------------------------------------------------------------------
-# $Id: forfile.tcl,v 8.2 1997/08/23 18:55:21 markd Exp $
+# $Id$
 #------------------------------------------------------------------------------
 #
 
 #@package: TclX-forfile for_file
 
 proc for_file {var filename cmd} {
-    upvar $var line
+    upvar 1 $var line
     set fp [open $filename r]
-    set code 0
-    set result {}
-    while {[gets $fp line] >= 0} {
-        set code [catch {uplevel 1 $cmd} result]
-        if {$code != 0 && $code != 4} break
+    try_eval {
+        set code 0
+        set result {}
+        while {[gets $fp line] >= 0} {
+            set code [catch {uplevel 1 $cmd} result]
+            if {$code != 0 && $code != 4} break
+        }
+    } {} {
+        close $fp
     }
-    close $fp
 
     if {$code == 0 || $code == 3 || $code == 4} {
         return $result
