@@ -8,7 +8,7 @@
  * is not available on that platform.  This results in code with minimal
  * number of #ifdefs.
  *-----------------------------------------------------------------------------
- * Copyright 1996-1997 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1996-1999 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -17,7 +17,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXunixOS.c,v 8.7 1997/08/08 16:51:04 markd Exp $
+ * $Id$
  *-----------------------------------------------------------------------------
  */
 
@@ -429,7 +429,11 @@ TclXOSsystem (interp, command, exitCode)
     }
     close (errPipes [0]);
 
-    waitpid (pid, (int *) &waitStatus, 0);
+    if (waitpid (pid, (int *) &waitStatus, 0) < 0) {
+        TclX_AppendObjResult (interp, "wait failed: ",
+                              Tcl_PosixError (interp), (char *) NULL);
+        return TCL_ERROR;
+    }
     
     /*
      * Return status based on wait result.
